from dna_features_viewer import BiopythonTranslator


class CustomTranslator(BiopythonTranslator):

    # Label fields indicates the order in which annotations fields are
    # considered to determine the feature's label
    label_fields = ["label", "note", "name", "gene"]

    def compute_feature_legend_text(self, feature):
        return feature.type
    
    def compute_feature_color(self, feature):
        return {
            "rep_origin": "yellow",
            "CDS": "#ffd383",  # light orange
            "regulatory": "red",
            "misc_recomb": "#fbf3f6",  # pink
            "misc_feature": "#d1e9f1",  # light blue
            "backbone": "darkblue",
        }[feature.type]
    
    def compute_feature_box_color(self, feature):
        return "white"
    
    def compute_feature_box_linewidth(self, feature):
        return 0


translator = CustomTranslator()
graphic_record = translator.translate_record("plasmid.gb")
ax, _ = graphic_record.plot(figure_width=13, strand_in_label_threshold=7)
graphic_record.plot_legend(ax=ax, loc=1, ncol=3, frameon=False)
ax.figure.savefig("A_linear_plot.svg", bbox_inches="tight")



from ..biotools import extract_graphical_translation


class SequenceAndTranslationMixin:
    def plot_sequence(
        self, ax, location=None, y_offset=1, fontdict=None, guides_intensity=0
    ):
        """Plot a sequence of nucleotides at the bottom of the plot.

        Parameters
        ----------

        ax
          Which axes the translation should be plotted to.

        location
          location of the segment to translate, either (start, end) or
          (start, end, strand).

        y_offset
          Number of text levels under the plot's base line where to draw the
          nucleotides. Should be 1 if the nucleotide sequence is to be plotted
          directly under the main line.

        fontdict
          Matplotlib fontdict for the text, e.g.
          ``{'size': 11, 'weight':'bold'}``

        background
          tuple (color1, color2) of alternate colors to plot behind each
          nucleotide position to guide vision. Leave to None for no background.

        guides_intensity
          Intensity of the vertical guides marking the different nucleotides
          (0 = no guides).
        """
        if self.sequence is None:
            raise ValueError("No sequence in the graphic record")
        if location is None:
            location = self.span
        location_start, location_end = location
        fontdict = dict(size=11)
        fontdict.update(fontdict or {})
        for i, nucleotide in enumerate(self.sequence):
            index = i + location_start
            if location_start <= index <= location_end:
                ax.text(
                    index,
                    -0.7 * self.feature_level_height * y_offset,
                    nucleotide,
                    ha="center",
                    va="center",
                    fontdict=fontdict,
                )
        if guides_intensity:
            color = (0, 0, 0, guides_intensity)
            for i in range(location_start, location_end + 1):
                ax.axvline(i - 0.5, linewidth=0.1, color=color, zorder=-10000)
        ymin = ax.get_ylim()[0]
        if ymin < -500:
            ymin = 0
        ax.set_ylim(bottom=min(ymin, -y_offset * self.feature_level_height))

    def plot_translation(
        self,
        ax,
        location=None,
        y_offset=2,
        fontdict=None,
        guides_intensity=0.5,
        translation=None,
        long_form_translation=True,
    ):
        """Plot a sequence of amino-acids at the bottom of the plot.

        Parameters
        ----------

        ax
          Which axes the translation should be plotted to.

        location
          location of the segment to translate (start, end).

        y_offset
          Number of text levels under the plot's base line where to draw the
          amino acid names. Should be 2 if the nucleotide sequence is also
          plotted at level 1.

        fontdict
          Matplotlib fontdict for the text, e.g.
          ``{'size': 11, 'weight':'bold'}``

        background
          tuple (color1, color2) of alternate colors to plot behind each
          amino acid position to guide vision. Leave to None for no background.

        translation
          Sequence of amino acids either as a string ``'MAKG...'`` or as a list
          ``['Met', 'Ala', ...]``
        """
        start, end = location[0], location[1]
        strand = location[2] if (len(location) == 3) else 1
        s, e = self.span
        start = max(start, s + ((start - s) % 3))
        end = min(end, e - ((end - e) % 3))
        if translation is None:
            new_loc = start - self.first_index, end - self.first_index, strand
            translation = extract_graphical_translation(
                self.sequence, location=new_loc, long_form=long_form_translation,
            )
        texts = [
            ((start + 3 * i, start + 3 * (i + 1)), aa)
            for i, aa in enumerate(translation)
        ]

        y = -0.7 * y_offset * self.feature_level_height
        ymin = ax.get_ylim()[0]
        ax.set_ylim(bottom=min(ymin, -y_offset * self.feature_level_height))
        fontdict = fontdict or {}
        guides_color = (0, 0, 0, guides_intensity)
        for i, ((start, end), text) in enumerate(texts):
            ax.text(
                0.5 * (start + end - 1),
                y,
                text,
                ha="center",
                va="center",
                fontdict=fontdict,
            )
            if guides_intensity:
                ax.axvline(
                    start - 0.5, linewidth=0.1, color=guides_color, zorder=-10000,
                )
        if guides_intensity:
            ax.axvline(end - 0.5, linewidth=0.1, color=guides_color, zorder=-10000)



from dna_features_viewer import BlackBoxlessLabelTranslator, load_record
import matplotlib.pyplot as plt
import numpy as np


class GCIndicatingTranslator(BlackBoxlessLabelTranslator):
    def compute_feature_legend_text(self, feature):
        if feature.qualifiers["gc%"] < 30:
            return "GC < 30%"
        elif feature.qualifiers["gc%"] < 60:
            return "30-60% GC"
        else:
            return "GC > 60%"

    def compute_feature_color(self, feature):
        return {
            "GC < 30%": "peachpuff",
            "30-60% GC": "azure",
            "GC > 60%": "skyblue",
        }[self.compute_feature_legend_text(feature)]

    def compute_feature_fontdict(self, feature):
        return dict(size=10, weight="bold", color="#494949")

    def compute_feature_label(self, feature):
        if not (30 < feature.qualifiers["gc%"] < 60):
            normal_label = super().compute_feature_label(feature)
            return normal_label + "-%d%%" % feature.qualifiers["gc%"]


def gc_content(sequence):
    return 100.0 * len([c for c in sequence if c in "GC"]) / len(sequence)


# DISPLAY THE SEQUENCE MAP

fig, (ax1, ax2) = plt.subplots(
    2, 1, figsize=(6, 3.5), sharex=True, gridspec_kw={"height_ratios": [3, 1]},
)
record = load_record("plasmid.gb")
for feature in record.features:
    feature.qualifiers["gc%"] = gc_content(feature.location.extract(record))
translator = GCIndicatingTranslator()
graphic_record = translator.translate_record(record)

graphic_record.plot(ax=ax1, with_ruler=False, strand_in_label_threshold=7)
graphic_record.plot_legend(ax=ax1, loc=1, frameon=False)

# DISPLAY THE GC% PROFILE ALONG THE SEQUENCE

window_size = 50
windowed_gc_content = [
    gc_content(record.seq[i : i + window_size])
    for i in range(len(record.seq) - window_size)
]
indices = np.arange(len(record.seq) - window_size) + 25

ax2.fill_between(indices[::50], windowed_gc_content[::50], alpha=0.3)
ax2.set_ylabel("GC(%)", fontsize=14)
ax2.set_ylim(bottom=0)

# SAVE THE FIGURE

fig.tight_layout()
fig.savefig("D_display_with_gc_content.svg", bbox_inches="tight")



from matplotlib import rc_context
from A_linear_plot import CustomTranslator

rc_context({"path.sketch": (1.5, 300, 1)})  # scale, length, randomness


class CustomTranslatorVariant(CustomTranslator):
    def compute_feature_fontdict(self, feature):
        return {"family": "Walter Turncoat"}


translator = CustomTranslatorVariant()
graphic_record = translator.translate_record("plasmid.gb")
cropped_record = graphic_record.crop((0, 1850))
ax, _ = cropped_record.plot(figure_width=2.5, with_ruler=False)
ax.figure.savefig("E_cartoon_plot.png", dpi=300, bbox_inches="tight")



from Bio import Entrez, SeqIO
from dna_features_viewer import annotate_biopython_record

Entrez.email = "dna_features_viewer@example.com"
handle = Entrez.efetch(
    db="nucleotide", id=1473096477, rettype="gb", retmode="text"
)
record = SeqIO.read(handle, "genbank")
annotate_biopython_record(
    record, location=(40, 1800), feature_type="backbone", label="backbone"
)
record.features = [
    f for f in record.features if f.type not in ["gene", "source"]
]
SeqIO.write(record, "plasmid.gb", "genbank")


from A_linear_plot import CustomTranslator

translator = CustomTranslator()
graphic_record = translator.translate_record("./plasmid.gb")
graphic_record.plot_on_multiple_pages(
    "multiline_plot.pdf",
    nucl_per_line=80,
    lines_per_page=10,
    plot_sequence=True
)



import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages
import numpy


class MultilinePlottableMixin:
    def plot_on_multiple_lines(
        self,
        n_lines=None,
        nucl_per_line=None,
        plot_sequence=False,
        figure_width="auto",
        **plot_params
    ):
        """Plot the features on different lines (one Matplotlib ax per line).

        Parameters
        ----------

        n_lines
          Number of lines on which the record will be plotted. A number of
          nucleotides per line can be provided instead (see below).

        nucl_per_line
          Number of nucleotides to be represented on every line (determines
          the number of lines ``n_lines``).

        plot_sequence
          Whether to plot the nucleotide sequence on each line.

        figure_width
          Width of the figure in inches. Leave to auto for a width of either 10
          (if not sequence is plotted) or 0.15*nucl_per_line inches
          (if a sequence is plotted).

        **plot_params
          Parameters from ``graphic_record.plot()`` to be used in the plotting
          of the individual lines. This includes ``draw_line``, ``with_ruler``,
          ``annotate_inline``, ``plot_sequence``,
          ``evelate_outline_annotations``, ``strand_in_label_pixel_threshold``.

        Returns
        -------

        figure, axes
          The matplotlib figure and axes generated.
        """

        if n_lines is None:
            n_lines = int(numpy.ceil(self.sequence_length / nucl_per_line))
        else:
            nucl_per_line = self.sequence_length // n_lines + 1

        if figure_width == "auto":
            if plot_sequence:
                figure_width = 0.15 * nucl_per_line
            else:
                figure_width = 10

        figures_heights = []

        def plot_line(line_index, ax=None):
            first, last = self.first_index, self.last_index
            line_start = first + line_index * nucl_per_line
            line_virtual_end = first + (line_index + 1) * nucl_per_line
            line_end = min(last, line_virtual_end)
            line_record = self.crop((line_start, line_end))
            line_ax, _ = line_record.plot(
                figure_width=figure_width,
                x_lim=(line_start, line_virtual_end),
                ax=ax,
                plot_sequence=plot_sequence,
                **plot_params
            )
            return line_ax

        for line_index in range(n_lines):
            line_ax = plot_line(line_index)
            figures_heights.append(line_ax.figure.get_figheight())
            plt.close(line_ax.figure)
        fig, axes = plt.subplots(
            n_lines,
            1,
            gridspec_kw={"height_ratios": figures_heights},
            figsize=(figure_width, 0.9 * sum(figures_heights)),
        )
        if n_lines == 1:
            axes = [axes]
        for line_index, ax in enumerate(axes):
            plot_line(line_index, ax=ax)
        fig.tight_layout()
        return fig, axes

    def plot_on_multiple_pages(
        self,
        pdf_target,
        n_lines=None,
        nucl_per_line=None,
        lines_per_page=5,
        figure_width="auto",
        **plot_params
    ):
        """Plot the features on different lines on different pages of a PDF.

        This function returns None

        Parameters
        ----------

        pdf_target
          Either a path to a PDF, or a file(-like) handle.

        n_lines
          Number of lines on which the record will be plotted. A number of
          nucleotides per line can be provided instead (see below).

        nucl_per_line
          Number of nucleotides to be represented on every line (determines
          the number of lines ``n_lines``).

        lines_per_page
          Number of lines on each page.

        plot_sequence
          Whether to plot the nucleotide sequence on each line.

        figure_width
          Width of the figure in inches. Leave to auto for a width of either 10
          (if not sequence is plotted) or 0.15*nucl_per_line inches
          (if a sequence is plotted).

        **plot_params
          Parameters from ``graphic_record.plot()`` to be used in the plotting
          of the individual lines. This includes ``draw_line``, ``with_ruler``,
          ``annotate_inline``, ``plot_sequence``,
          ``evelate_outline_annotations``, ``strand_in_label_pixel_threshold``.
        """
        nucl_per_page = nucl_per_line * lines_per_page
        number_of_pages = int(numpy.ceil(self.sequence_length / nucl_per_page))
        with PdfPages(pdf_target) as pdf:
            for page_index in range(number_of_pages):
                first, last = self.first_index, self.last_index
                page_start = first + page_index * nucl_per_page
                page_end = first + (page_index + 1) * nucl_per_page
                page_end = min(last, page_end)
                page_record = self.crop((page_start, page_end))
                fig, axes = page_record.plot_on_multiple_lines(
                    nucl_per_line=nucl_per_line,
                    figure_width=figure_width,
                    **plot_params
                )
                pdf.savefig(fig)
                plt.close(fig)
